
import sys
import re

def read_corpus():
    out = []
    with open("all_sentiment_shuffled.txt") as f:
        for line in f:            
            tokens = line.strip().split()
            out.append( (tokens[0], tokens[1], tokens[2], tokens[3:]) )
    return out

def train_by_errors(labeled_documents, categories, number_iterations):
    weights = {}    
    for iteration in range(number_iterations):
        print "Iteration %s." % iteration
        for label, _, _, document in labeled_documents: 
            guess = guess_category(document, weights, categories)
            if label != guess:
                for word in document:
                    weights[(word, label)] = weights.get((word,label), 0) + 1
                    weights[(word, guess)] = weights.get((word,guess), 0) - 1
    return weights

def score_category(document, weights, category):
    score = 0    
    for word in document:
        score += weights.get((word, category), 0)    
    return score

def guess_category(document, weights, categories):
    return max(categories, key=lambda c: score_category(document, weights, c))

def evaluate(labeled_documents, weights, categories):
    ncorrect = 0
    for label, _, _, document in labeled_documents: 
        guess = guess_category(document, weights, categories)
        if guess == label:
            ncorrect += 1
    return float(ncorrect)/len(labeled_documents)

def print_weights_sorted(weights, categories):
    ws = sorted(weights, key=weights.get, reverse=True)
    for c in categories:
        ws_c = [ w for (w, cc) in ws if c == cc ][:20]
        print c + ":"
        for w in ws_c:
            print "", w, weights[(w, c)]

def get_categories(labeled_documents):
    return list(set(label for label, _, _, _ in labeled_documents))

all_docs = read_corpus()

train_docs = all_docs[:len(all_docs)/2]
eval_docs = all_docs[len(all_docs)/2:]

categories = get_categories(train_docs)

weights = train_by_errors(train_docs, categories, 20)

print evaluate(eval_docs, weights, categories)

#print_weights_sorted(weights, categories)
