
import sys
import re

mpqa_re = re.compile("([^ =]+)=([^ ]+)")

def read_corpus():
    out = []
    with open("all_sentiment_shuffled.txt") as f:
        for line in f:            
            tokens = line.strip().split()
            out.append( (tokens[0], tokens[1], tokens[2], tokens[3:]) )
    return out

def parse_mpqa_line(line):
    fs = {}
    for m in mpqa_re.finditer(line.strip()):
        fs[m.group(1)] = m.group(2)
    return fs

def read_mpqa_lexicon():
    weights = {}
    with open("mpqa.txt") as f:
        for line in f:
            fs = parse_mpqa_line(line)
            t = fs["type"]
            w = fs["word1"]
            p = fs["priorpolarity"]
            if type == "strongsubj":
                weight = 2
            else:
                weight = 1
            if p == "negative":
                weight *= -1
            elif p == "positive":
                pass
            else:
                weight = 0
            weights[w] = weight
    return weights

def train_by_errors(labeled_documents, number_iterations):
    weights = {}    
    for iteration in range(number_iterations):
        print "Iteration %s." % iteration
        for _, label, _, document in labeled_documents: 
            guess = guess_sentiment(document, weights)
            if label == "pos" and guess == "neg":
                for word in document:
                    weights[word] = weights.get(word, 0) + 1
            elif label == "neg" and guess == "pos":
                for word in document:
                    weights[word] = weights.get(word, 0) - 1
    return weights

def guess_sentiment(document, weights):
    score = 0
    for word in document:
        score += weights.get(word, 0)    
    if score >= 0:
        return "pos"
    else:
        return "neg"    

def evaluate(labeled_documents, weights):
    ncorrect = 0
    for _, label, _, document in labeled_documents: 
        guess = guess_sentiment(document, weights)
        if guess == label:
            ncorrect += 1
    return float(ncorrect) / len(labeled_documents)

def print_weights_sorted(weights):
    max_pos_words = sorted(weights, key=weights.get, reverse=True)[:20]
    max_neg_words = sorted(weights, key=weights.get, reverse=False)[:20]
    
    print "Positive words:"
    for word in max_pos_words:
        print "", word, weights[word]
    print "Negative words:"
    for word in max_neg_words:
        print "", word, weights[word]

all_docs = read_corpus()

train_docs = all_docs[:len(all_docs)/2]
eval_docs = all_docs[len(all_docs)/2:]

#weights = read_mpqa_lexicon()
weights = train_by_errors(train_docs, 20)

print evaluate(eval_docs, weights)

#print_weights_sorted(weights)
