
import numpy

from sklearn.feature_extraction import DictVectorizer
from sklearn.linear_model import Perceptron

def read_corpus():
    out = []
    with open("all_sentiment_shuffled.txt") as f:
        for line in f:            
            tokens = line.strip().split()
            out.append( (tokens[0], tokens[1], tokens[2], tokens[3:]) )
    return out

def encode_corpus(labeled_documents):
    data = []
    target = []
    for _, label, _, document in labeled_documents:
        feature_dict = {}
        for w in document:
            feature_dict[w] = True
        data.append(feature_dict)
        target.append(label)
    return data, target

def experiment():
    all_docs = read_corpus()
    data, target = encode_corpus(all_docs)
    train_docs = data[:len(all_docs)/2]
    train_targets = target[:len(all_docs)/2]
    eval_docs = data[len(all_docs)/2:]
    eval_targets = target[len(all_docs)/2:]

    vec = DictVectorizer()
    clf = Perceptron(n_iter=20)
    clf.fit(vec.fit_transform(train_docs),
            numpy.array(train_targets))

    n_correct = 0
    for doc, label in zip(eval_docs, eval_targets):
        guess = clf.predict(vec.transform(doc))
        if guess == label:
            n_correct += 1
    print float(n_correct) / len(eval_docs)

if __name__ == "__main__":
    experiment()
    
